#region Copyright  Marian Placko 2010-2011
//
// All rights are reserved. Reproduction or transmission in whole or in part,
// in any form or by any means, electronic, mechanical or otherwise, is
// prohibited without the prior written consent of the copyright owner.
//
#endregion

using System;
using System.Reflection;
using System.Windows.Forms;
using EnvDTE;
using EnvDTE80;
using Microsoft.SqlServer.Management.Common;
using Microsoft.SqlServer.Management.SqlStudio.Explorer; // NOTE: SQL Server 2012!
using Microsoft.SqlServer.Management.Smo.RegSvrEnum;
using Microsoft.SqlServer.Management.UI.VSIntegration;
using Microsoft.SqlServer.Management.UI.VSIntegration.ObjectExplorer;

namespace MarianPlacko.SqlSmartSecurity
{
    /// <summary>
    /// The class of a form.
    /// </summary>
    public class Form
    {
        /// <summary>
        /// The object of a current instance.
        /// </summary>
        private static readonly Form CurrentInstance = new Form();

        /// <summary>
        /// The object of a current connection.
        /// </summary>
        private SqlConnectionInfo _currentConnection;

        /// <summary>
        /// The object of a current UI connection.
        /// </summary>
        private UIConnectionInfo _currentUiConnection;

        /// <summary>
        /// The object property of a form.
        /// </summary>
        public static Form Instance
        {
            get { return CurrentInstance; }
        }

        /// <summary>
        /// The object property of a window.
        /// </summary>
        [CLSCompliantAttribute(false)]
        public Window Window { get; private set; }

        /// <summary>
        /// Get the active window connection.
        /// </summary>
        public SqlConnectionInfo GetActiveWindowConnection
        {
            get
            {
                SqlConnectionInfo info = null;
                try
                {
                    UIConnectionInfo connInfo = null;
                    if (ServiceCache.ScriptFactory.CurrentlyActiveWndConnectionInfo != null)
                    {
                        connInfo = ServiceCache.ScriptFactory.CurrentlyActiveWndConnectionInfo.UIConnectionInfo;
                    }
                    if (connInfo != null)
                    {
                        if (connInfo == _currentUiConnection)
                        {
                            return _currentConnection;
                        }
                        info = CreateSqlConnectionInfo(connInfo);
                        _currentConnection = info;
                        _currentUiConnection = connInfo;
                    }
                    if (info == null)
                    {
                        var nodes = GetObjectExplorerSelectedNodes();
                        if (nodes.Length > 0) // NOTE: attempt to get connection info from node in tree view
                        {
                            info = nodes[0].Connection as SqlConnectionInfo;
                        }
                    }
                    return info;
                }
                catch (NullReferenceException)
                {
                    return null;
                }
            }
        }

        /// <summary>
        /// Get the object explorer selected nodes.
        /// </summary>
        /// <returns>Unknown.</returns>
        private static INodeInformation[] GetObjectExplorerSelectedNodes()
        {
            //var objExplorer = ServiceCache.GetObjectExplorer(); // NOTE: SQL Server 2005/2008!
            // NOTE: SQL Server 2012!
            var objExplorer =
                (ObjectExplorerService) ServiceCache.ServiceProvider.GetService(typeof (IObjectExplorerService));
            int arraySize;
            INodeInformation[] nodes;
            objExplorer.GetSelectedNodes(out arraySize, out nodes);
            return nodes;
        }

        /// <summary>
        /// Create the SQL connection info.
        /// </summary>
        /// <param name="connectionInfo">Unknown.</param>
        /// <returns>Unknown.</returns>
        private static SqlConnectionInfo CreateSqlConnectionInfo(UIConnectionInfo connectionInfo)
        {
            var sqlConnInfo = new SqlConnectionInfo
                                  {
                                      ServerName = connectionInfo.ServerName,
                                      UserName = connectionInfo.UserName
                                  };
            if (string.IsNullOrEmpty(connectionInfo.Password)) // NOTE: this is a bit of a hack
            {
                sqlConnInfo.UseIntegratedSecurity = true;
            }
            else
            {
                sqlConnInfo.Password = connectionInfo.Password;
            }
            return sqlConnInfo;
        }

        /// <summary>
        /// Create the Add-in window.
        /// </summary>
        /// <param name="addInInstance">The instance of an Add-in.</param>
        [CLSCompliantAttribute(false)]
        public void CreateAddInWindow(AddIn addInInstance)
        {
            var id = new Guid("4c410c93-d66b-495a-9de2-99d5bde4a3b9"); // NOTE: id == random guid
            var asm = Assembly.GetExecutingAssembly();
            Window = CreateToolWindow(Resources.Resources._companyName + ".SqlSmartSecurity.Encryption", asm.Location, id,
                                      addInInstance);
        }

        /// <summary>
        /// Create the tool window.
        /// </summary>
        /// <param name="typeName">Unknown.</param>
        /// <param name="assemblyLocation">Unknown.</param>
        /// <param name="uiTypeGuid">Unknown.</param>
        /// <param name="addinInstance">The instance of an Add-in.</param>
        /// <returns>Unknown.</returns>
        private static Window CreateToolWindow(string typeName, string assemblyLocation, Guid uiTypeGuid,
                                               AddIn addinInstance)
        {
            var win2 = ServiceCache.ExtensibilityModel.Windows as Windows2;
            if (win2 != null)
            {
                object controlObject = null;
                try
                {
                    var toolWindow = win2.CreateToolWindow2(addinInstance, assemblyLocation, typeName,
                                                            Resources.Resources._logoName + " " +
                                                            Resources.Resources._productName, "{" + uiTypeGuid + "}",
                                                            ref controlObject);
                    toolWindow.Visible = true;
                    return toolWindow;
                }
                catch // NOTE: SQL Server 2012!
                {
                    try
                    {
                        var toolWindow = win2.CreateToolWindow2(addinInstance, assemblyLocation, typeName,
                                                                Resources.Resources._logoName + " " +
                                                                Resources.Resources._productName, "{" + uiTypeGuid + "}",
                                                                ref controlObject);
                        toolWindow.Visible = true;
                        return toolWindow;
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message,
                                Resources.Resources._3S_errMsgBoxCaption, MessageBoxButtons.OK,
                                MessageBoxIcon.Error, MessageBoxDefaultButton.Button1,
                                MessageBoxOptions.DefaultDesktopOnly);
                    }
                }
            }
            return null;
        }
    }
}